@@ -13,7 +13,9 @@
  *
  * Known limitations:
  * - psutil.Process.io_counters read count is always 0
+ * - psutil.Process.io_counters may not be available on older AIX versions
  * - psutil.Process.threads may not be available on older AIX versions
+ * - psutil.net_io_counters may not be available on older AIX versions
  * - reading basic process info may fail or return incorrect values when
  *   process is starting (see IBM APAR IV58499 - fixed in newer AIX versions)
  * - sockets and pipes may not be counted in num_fds (fixed in newer AIX
@@ -21,20 +23,20 @@
  *
  * Useful resources:
  * - proc filesystem: http://www-01.ibm.com/support/knowledgecenter/
- *       ssw_aix_61/com.ibm.aix.files/proc.htm
+ *       ssw_aix_72/com.ibm.aix.files/proc.htm
  * - libperfstat: http://www-01.ibm.com/support/knowledgecenter/
- *       ssw_aix_61/com.ibm.aix.files/libperfstat.h.htm
+ *       ssw_aix_72/com.ibm.aix.files/libperfstat.h.htm
  */
 
 #include <Python.h>
-
-#include <sys/types.h>
-#include <sys/stat.h>
+#include <sys/limits.h>
 #include <sys/proc.h>
-#include <sys/sysinfo.h>
 #include <sys/procfs.h>
 #include <sys/socket.h>
+#include <sys/stat.h>
+#include <sys/sysinfo.h>
 #include <sys/thread.h>
+#include <sys/types.h>
 #include <fcntl.h>
 #include <utmp.h>
 #include <utmpx.h>
@@ -48,11 +50,11 @@
 #include <libperfstat.h>
 #include <unistd.h>
 
+#include "_psutil_common.h"
+#include "_psutil_posix.h"
 #include "arch/aix/ifaddrs.h"
 #include "arch/aix/net_connections.h"
 #include "arch/aix/common.h"
-#include "_psutil_common.h"
-#include "_psutil_posix.h"
 
 
 #define TV2DOUBLE(t)   (((t).tv_nsec * 0.000000001) + (t).tv_sec)
@@ -132,17 +134,14 @@ psutil_proc_basic_info(PyObject *self, PyObject *args) {
 
 
 /*
- * Return process name and args as a Python tuple.
+ * Return process name as a Python string.
  */
 static PyObject *
-psutil_proc_name_and_args(PyObject *self, PyObject *args) {
+psutil_proc_name(PyObject *self, PyObject *args) {
     int pid;
     char path[100];
     psinfo_t info;
     const char *procfs_path;
-    PyObject *py_name = NULL;
-    PyObject *py_args = NULL;
-    PyObject *py_retlist = NULL;
 
     if (! PyArg_ParseTuple(args, "is", &pid, &procfs_path))
         return NULL;
@@ -150,28 +149,143 @@ psutil_proc_name_and_args(PyObject *self, PyObject *args) {
     if (! psutil_file_to_struct(path, (void *)&info, sizeof(info)))
         return NULL;
 
-    py_name = PyUnicode_DecodeFSDefault(info.pr_fname);
-    if (!py_name)
+    return PyUnicode_DecodeFSDefaultAndSize(info.pr_fname, PRFNSZ);
+}
+
+
+/*
+ * Return process command line arguments as a Python list
+ */
+static PyObject *
+psutil_proc_args(PyObject *self, PyObject *args) {
+    int pid;
+    PyObject *py_retlist = PyList_New(0);
+    PyObject *py_arg = NULL;
+    struct procsinfo procbuf;
+    long arg_max;
+    char *argbuf = NULL;
+    char *curarg = NULL;
+    int ret;
+
+    if (py_retlist == NULL)
+        return NULL;
+    if (!PyArg_ParseTuple(args, "i", &pid))
         goto error;
-    py_args = PyUnicode_DecodeFSDefault(info.pr_psargs);
-    if (!py_args)
+    arg_max = sysconf(_SC_ARG_MAX);
+    argbuf = malloc(arg_max);
+    if (argbuf == NULL) {
+        PyErr_NoMemory();
         goto error;
-    py_retlist = Py_BuildValue("OO", py_name, py_args);
-    if (!py_retlist)
+    }
+
+    procbuf.pi_pid = pid;
+    ret = getargs(&procbuf, sizeof(procbuf), argbuf, ARG_MAX);
+    if (ret == -1) {
+        PyErr_SetFromErrno(PyExc_OSError);
         goto error;
-    Py_DECREF(py_name);
-    Py_DECREF(py_args);
+    }
+
+    curarg = argbuf;
+    /* getargs will always append an extra NULL to end the arg list,
+     * even if the buffer is not big enough (even though it is supposed
+     * to be) so the following 'while' is safe */
+    while (*curarg != '\0') {
+        py_arg = PyUnicode_DecodeFSDefault(curarg);
+        if (!py_arg)
+            goto error;
+        if (PyList_Append(py_retlist, py_arg))
+            goto error;
+        Py_DECREF(py_arg);
+        curarg = strchr(curarg, '\0') + 1;
+    }
+
+    free(argbuf);
+
     return py_retlist;
 
 error:
-    Py_XDECREF(py_name);
-    Py_XDECREF(py_args);
+    if (argbuf != NULL)
+        free(argbuf);
     Py_XDECREF(py_retlist);
+    Py_XDECREF(py_arg);
+    return NULL;
+}
+
+
+/*
+ * Return process environment variables as a Python dict
+ */
+static PyObject *
+psutil_proc_environ(PyObject *self, PyObject *args) {
+    int pid;
+    PyObject *py_retdict = PyDict_New();
+    PyObject *py_key = NULL;
+    PyObject *py_val = NULL;
+    struct procsinfo procbuf;
+    long env_max;
+    char *envbuf = NULL;
+    char *curvar = NULL;
+    char *separator = NULL;
+    int ret;
+
+    if (py_retdict == NULL)
+        return NULL;
+    if (!PyArg_ParseTuple(args, "i", &pid))
+        goto error;
+    env_max = sysconf(_SC_ARG_MAX);
+    envbuf = malloc(env_max);
+    if (envbuf == NULL) {
+        PyErr_NoMemory();
+        goto error;
+    }
+
+    procbuf.pi_pid = pid;
+    ret = getevars(&procbuf, sizeof(procbuf), envbuf, ARG_MAX);
+    if (ret == -1) {
+        PyErr_SetFromErrno(PyExc_OSError);
+        goto error;
+    }
+
+    curvar = envbuf;
+    /* getevars will always append an extra NULL to end the arg list,
+     * even if the buffer is not big enough (even though it is supposed
+     * to be) so the following 'while' is safe */
+    while (*curvar != '\0') {
+        separator = strchr(curvar, '=');
+        if (separator != NULL) {
+            py_key = PyUnicode_DecodeFSDefaultAndSize(
+                curvar,
+                (Py_ssize_t)(separator - curvar)
+            );
+            if (!py_key)
+                goto error;
+            py_val = PyUnicode_DecodeFSDefault(separator + 1);
+            if (!py_val)
+                goto error;
+            if (PyDict_SetItem(py_retdict, py_key, py_val))
+                goto error;
+            Py_CLEAR(py_key);
+            Py_CLEAR(py_val);
+        }
+        curvar = strchr(curvar, '\0') + 1;
+    }
+
+    free(envbuf);
+
+    return py_retdict;
+
+error:
+    if (envbuf != NULL)
+        free(envbuf);
+    Py_XDECREF(py_retdict);
+    Py_XDECREF(py_key);
+    Py_XDECREF(py_val);
     return NULL;
 }
 
 
 #ifdef CURR_VERSION_THREAD
+
 /*
  * Retrieves all threads used by process returning a list of tuples
  * including thread id, user time and system time.
@@ -237,9 +351,12 @@ error:
         free(threadt);
     return NULL;
 }
+
 #endif
 
 
+#ifdef CURR_VERSION_PROCESS
+
 static PyObject *
 psutil_proc_io_counters(PyObject *self, PyObject *args) {
     long pid;
@@ -263,6 +380,8 @@ psutil_proc_io_counters(PyObject *self, PyObject *args) {
                          procinfo.outBytes);
 }
 
+#endif
+
 
 /*
  * Return process user and system CPU times as a Python tuple.
@@ -342,7 +461,7 @@ psutil_proc_num_ctx_switches(PyObject *self, PyObject *args) {
 
     /* finished iteration without finding requested pid */
     free(processes);
-    return NoSuchProcess("");
+    return NoSuchProcess("psutil_read_process_table (no PID found)");
 }
 
 
@@ -390,10 +509,10 @@ psutil_users(PyObject *self, PyObject *args) {
             goto error;
         if (PyList_Append(py_retlist, py_tuple))
             goto error;
-        Py_DECREF(py_username);
-        Py_DECREF(py_tty);
-        Py_DECREF(py_hostname);
-        Py_DECREF(py_tuple);
+        Py_CLEAR(py_username);
+        Py_CLEAR(py_tty);
+        Py_CLEAR(py_hostname);
+        Py_CLEAR(py_tuple);
     }
     endutxent();
 
@@ -450,9 +569,9 @@ psutil_disk_partitions(PyObject *self, PyObject *args) {
             goto error;
         if (PyList_Append(py_retlist, py_tuple))
             goto error;
-        Py_DECREF(py_dev);
-        Py_DECREF(py_mountp);
-        Py_DECREF(py_tuple);
+        Py_CLEAR(py_dev);
+        Py_CLEAR(py_mountp);
+        Py_CLEAR(py_tuple);
         mt = getmntent(file);
     }
     endmntent(file);
@@ -469,6 +588,8 @@ error:
 }
 
 
+#if defined(CURR_VERSION_NETINTERFACE) && CURR_VERSION_NETINTERFACE >= 3
+
 /*
  * Return a list of tuples for network I/O statistics.
  */
@@ -538,6 +659,8 @@ error:
     return NULL;
 }
 
+#endif
+
 
 static PyObject*
 psutil_net_if_stats(PyObject* self, PyObject* args) {
@@ -556,7 +679,7 @@ psutil_net_if_stats(PyObject* self, PyObject* args) {
     if (sock == -1)
         goto error;
 
-    strncpy(ifr.ifr_name, nic_name, sizeof(ifr.ifr_name));
+    PSUTIL_STRNCPY(ifr.ifr_name, nic_name, sizeof(ifr.ifr_name));
 
     // is up?
     ret = ioctl(sock, SIOCGIFFLAGS, &ifr);
@@ -753,7 +876,7 @@ error:
 static PyObject *
 psutil_virtual_mem(PyObject *self, PyObject *args) {
     int rc;
-    int pagesize = getpagesize();
+    long pagesize = psutil_getpagesize();
     perfstat_memory_total_t memory;
 
     rc = perfstat_memory_total(
@@ -779,7 +902,7 @@ psutil_virtual_mem(PyObject *self, PyObject *args) {
 static PyObject *
 psutil_swap_mem(PyObject *self, PyObject *args) {
     int rc;
-    int pagesize = getpagesize();
+    long pagesize = psutil_getpagesize();
     perfstat_memory_total_t memory;
 
     rc = perfstat_memory_total(
@@ -868,8 +991,12 @@ PsutilMethods[] =
     // --- process-related functions
     {"proc_basic_info", psutil_proc_basic_info, METH_VARARGS,
      "Return process ppid, rss, vms, ctime, nice, nthreads, status and tty"},
-    {"proc_name_and_args", psutil_proc_name_and_args, METH_VARARGS,
-     "Return process name and args."},
+    {"proc_name", psutil_proc_name, METH_VARARGS,
+     "Return process name."},
+    {"proc_args", psutil_proc_args, METH_VARARGS,
+     "Return process command line arguments."},
+    {"proc_environ", psutil_proc_environ, METH_VARARGS,
+     "Return process environment variables."},
     {"proc_cpu_times", psutil_proc_cpu_times, METH_VARARGS,
      "Return process user and system CPU times."},
     {"proc_cred", psutil_proc_cred, METH_VARARGS,
@@ -878,8 +1005,10 @@ PsutilMethods[] =
     {"proc_threads", psutil_proc_threads, METH_VARARGS,
      "Return process threads"},
 #endif
+#ifdef CURR_VERSION_PROCESS
     {"proc_io_counters", psutil_proc_io_counters, METH_VARARGS,
      "Get process I/O counters."},
+#endif
     {"proc_num_ctx_switches", psutil_proc_num_ctx_switches, METH_VARARGS,
      "Get process I/O counters."},
 
@@ -898,8 +1027,10 @@ PsutilMethods[] =
      "Return system virtual memory usage statistics"},
     {"swap_mem", psutil_swap_mem, METH_VARARGS,
      "Return stats about swap memory, in bytes"},
+#if defined(CURR_VERSION_NETINTERFACE) && CURR_VERSION_NETINTERFACE >= 3
     {"net_io_counters", psutil_net_io_counters, METH_VARARGS,
      "Return a Python dict of tuples for network I/O statistics."},
+#endif
     {"net_connections", psutil_net_connections, METH_VARARGS,
      "Return system-wide connections"},
     {"net_if_stats", psutil_net_if_stats, METH_VARARGS,
